package ch.hepia.api.transport;

import org.json.JSONObject;

public class Coordinates {
    private String type;
    private double x;
    private double y;

    /**
     * Default coordinates containing default values (Geneva is the default value)
     * WGS84 is the default type value
     */
    private final static class DefaultCoordinates extends Coordinates{
        /**
         * Constructor of DefaultCoordinates, located in Geneva
         */
        private DefaultCoordinates(){
            super("WGS84", 46.210237, 6.142422);
        }
    }

    /**
     * Constructor of the class Coordinates
     * A coordinates is represented by a type of place and a point (x, y)
     * @param type type of place we are a these coordinates
     * @param x coordinate in x
     * @param y coordinate in y
     */
    private Coordinates(String type, double x, double y){
        this.type = type;
        this.x = x;
        this.y = y;
    }

    @Override
    public String toString(){
        StringBuilder string = new StringBuilder();
        return string.append("Coordonnées du type ").append(type).append(" au point (").append(x).append(", ").append(y).append(")").toString();
    }

    /**
     * Get the x coordinate of the place (x, y)
     * @return x
     */
    public double getX(){
        return x;
    }

    /**
     * Get the y coordinate of the place (x, y)
     * @return y
     */
    public double getY(){
        return y;
    }

    /**
     * Builder of Coordinates object
     */
    public final static class CoordinatesBuilder{
        private JSONObject datas;

        /**
         * Constructor of the class CoordinatesBuilder
         * Stock Datas from a JSON object given
         * @param datas contains datas to construct a Coordinates object
         */
        public CoordinatesBuilder(JSONObject datas){
            this.datas = datas;
        }

        /**
         * Build a Coordinates object from the datas obtained
         * @return a new Coordinates object
         */
        public Coordinates build(){
            if(datas.isEmpty()){
                return new DefaultCoordinates();
            }

            String type = datas.getString("type");
            double x = datas.getDouble("x");
            double y = datas.getDouble("y");

            return new Coordinates(type, x, y);
        }
    }
}
