package ch.hepia.api.transport;

import org.json.JSONArray;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;

public class LinkAPI {
    private String baseURL = "http://transport.opendata.ch/v1/";

    /**
     * Fetch datas with an http GET request on the form of a string given the url path
     * @param path
     * @return content
     * @throws IOException
     */
    private String getContent(String path) throws IOException {
        URL url = new URL(path);
        HttpURLConnection http = (HttpURLConnection) url.openConnection();
        http.setRequestMethod("GET");
        http.connect();

        BufferedReader reader = new BufferedReader(new InputStreamReader(http.getInputStream()));
        String line;
        StringBuilder content = new StringBuilder();
        while((line = reader.readLine()) != null){
            content.append(line);
        }
        reader.close();
        http.disconnect();
        return content.toString();
    }

    /**
     * Get the stations containing the string query in their names
     * This return an array of Station type Location Objects
     * @param query
     * @return a JSONArray
     * @throws IOException
     */
    public JSONArray getStations(String query) throws IOException {
        String path = baseURL + "locations?query=" + URLEncoder.encode(query, StandardCharsets.UTF_8);
        String content = getContent(path);

        return new JSONObject(content).getJSONArray("stations");
    }

    /**
     * Get all the connections from an end point to another in the form of a JSONArray
     * Return an Array of Connection Objects
     * @param from
     * @param to
     * @return a JSONArray
     * @throws IOException
     */
    public JSONArray getConnections(String from, String to) throws IOException {
        String path = baseURL + "connections?from=" +
                URLEncoder.encode(from, StandardCharsets.UTF_8) + "&to=" +
                URLEncoder.encode(to, StandardCharsets.UTF_8);
        String content = getContent(path);

        return new JSONObject(content).getJSONArray("connections");
    }

    /**
     * Get the Stationboard of the station given in a JSONArray
     * The StationBoard is an array of Journey Objects
     * The limit is the maximum number of stations the Stationboard should fetch
     * @param station
     * @param limit
     * @return
     * @throws IOException
     */
    public JSONArray getStationBoard(String station, int limit) throws IOException {
        String path = baseURL + "stationboard?station=" + URLEncoder.encode(station, StandardCharsets.UTF_8);
        if(limit != 0){
            path += "&limit=" + limit;
        }
        String content = getContent(path);

        return new JSONObject(content).getJSONArray("stationboard");
    }

}
