package ch.hepia.api.transport;

import org.json.JSONObject;

public class Location {
    private int id;
    private Type type;
    private String name;
    private double score;
    private Coordinates where;
    private double distance;

    /**
     * Default Location
     */
    private final static class DefaultLocation extends Location{
        /**
         * Constructor of DefaultLocation, centered in Geneva
         * Type is REFINE, because it's a default choice
         */
        private DefaultLocation(){
            //here building new empty coordinates will return a default coordinates corresponding to Geneva
            super(8501008, Type.REFINE, "Genève", 0, new Coordinates.CoordinatesBuilder(new JSONObject()).build(), 0);
        }
    }

    /**
     * Constructor of the class Location
     * @param id unique to the Location
     * @param type type of place the Location is
     * @param name
     * @param score accuracy of the result (may not be used)
     * @param where Localisation of the Location
     * @param distance distance to the Location if there has been Coordinates (may not be used)
     */
    private Location(int id, Type type, String name, double score, Coordinates where, double distance){
        this.id = id;
        this.type = type;
        this.name = name;
        this.score = score;
        this.where = where;
        this.distance = distance;
    }

    /**
     * Get the coordinates of the Location
     * @return where
     */
    public Coordinates getCoordinates(){
        return where;
    }

    /**
     * Builder of Location objects
     */
    public final static class LocationBuilder{
        private JSONObject datas;
        private String type;

        /**
         * Constructor of the class LocationBuilder
         * Stock Datas from a JSON object given
         * @param datas contains datas to construct a Locatiop object
         */
        public LocationBuilder(JSONObject datas, String type){
            this.datas = datas;
            this.type = type;
        }

        /**
         * Build a Location object with the datas obtained
         * @return a new Location object
         */
        public Location build(){
            if(datas.isEmpty()){
                return new DefaultLocation();
            }

            int id = datas.isNull("id") ? 0 : datas.getInt("id");
            Type type = Type.valueOf(this.type.toUpperCase());
            String name = datas.isNull("name") ? "" : datas.getString("name");
            double score = datas.isNull("score") ? 0 : datas.getDouble("score");

            JSONObject coordinatesJSON = datas.getJSONObject("coordinate");
            Coordinates.CoordinatesBuilder coordinates = new Coordinates.CoordinatesBuilder(coordinatesJSON);

            double distance = datas.isNull("distance") ? 0 : datas.getDouble("distance");

            return new Location(id, type, name, score, coordinates.build(), distance);
        }
    }
}
