package ch.hepia.api.transport;

import org.json.JSONObject;

import java.text.ParseException;

public class Section {
    private Journey journey;
    private double walktime;
    private Stop departure;
    private Stop arrival;

    /**
     * Empty Section
     */
    private final static class EmptySection extends Section{
        /**
         * Constructor of EmptySection
         * An empty Section is a Journey From Geneva to Geneva
         * @throws ParseException
         */
        private EmptySection() throws ParseException {
            super(new Journey.JourneyBuilder(new JSONObject()).build(), 0,
                    new Stop.StopBuilder(new JSONObject()).build(),
                    new Stop.StopBuilder(new JSONObject()).build());
        }
    }


    /**
     * Constructor of the class Section
     * @param journey what Journey this section is part of
     * @param walkTime time to get to the Section by walk
     * @param departure where the Section will depart from
     * @param arrival where the Section will arrive to
     */
    private Section(Journey journey, double walkTime, Stop departure, Stop arrival){
        this.journey = journey;
        this.walktime = walkTime;
        this.departure = departure;
        this.arrival = arrival;
    }

    /**
     * Get the journey this Section is part of
     * @return journey
     */
    public Journey getJourney(){
        return journey;
    }

    /**
     * Get the Stop where the Section will depart from
     * @return departure
     */
    public Stop getDeparture(){
        return departure;
    }

    /**
     * get the Stop where the Section is headed to
     * @return arrival
     */
    public Stop getArrival(){
        return arrival;
    }

    /**
     * Builder of Section objects
     */
    public final static class SectionBuilder{
        private JSONObject datas;

        /**
         * Constructor of the class SectionBuilder
         * Stock Datas from a JSON object given
         * @param datas contains datas to construct a Section object
         */
        public SectionBuilder(JSONObject datas){
            this.datas = datas;
        }

        /**
         * Build a Section object from the datas obtained
         * @return a new Section object
         * @throws ParseException
         */
        public Section build() throws ParseException {
            if(datas.isEmpty()){
                return new EmptySection();
            }

            JSONObject journeyJSON = datas.getJSONObject("journey");
            Journey.JourneyBuilder journey = new Journey.JourneyBuilder(journeyJSON);

            double walktime = datas.isNull("walktime") ? 0 : datas.getDouble("walktime");

            JSONObject departureJSON = datas.getJSONObject("departure");
            Stop.StopBuilder departure = new Stop.StopBuilder(departureJSON);

            JSONObject arrivalJSON = datas.getJSONObject("arrival");
            Stop.StopBuilder arrival = new Stop.StopBuilder(arrivalJSON);

            return new Section(journey.build(), walktime, departure.build(), arrival.build());
        }
    }
}
