package ch.hepia.api.transport;

import org.json.JSONObject;


public class Service {
    private String regular;
    private String irregular;

    /**
     * Empty Service
     */
    private final static class EmptyService extends Service{
        /**
         * Constructor of EmptyService
         * No informations given
         */
        private EmptyService(){
            super("", "");
        }
    }

    /**
     * Constructor of Service class
     * @param regular is the service regular ?
     * @param irregular is the service disturbed ?
     */
    private Service(String regular, String irregular){
        this.regular = regular;
        this.irregular = irregular;
    }

    /**
     * Builder of Service objects
     */
    public final static class ServiceBuilder{
        private JSONObject datas;

        /**
         * Constructor of the class ServiceBuilder
         * Stock Datas from a JSON object given
         * @param datas contains datas to construct a Service object
         */
        public ServiceBuilder(JSONObject datas){
            this.datas = datas;
        }

        /**
         * Build a Service object from the datas obtained
         * @return a new Service object
         */
        public Service build(){
            if(datas.isEmpty()){
                return new EmptyService();
            }

            String regular = datas.isNull("regular") ? "" : datas.getString("regular");
            String irregular = datas.isNull("irregular") ? "" : datas.getString("irregular");

            return new Service(regular, irregular);
        }
    }
}
